/*
 * Copyright 2012, RagingGoblin <http://raginggoblin.wordpress.com>
 * 
 * This file is part of Infolog.
 *
 *  Infolog is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Infolog is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Infolog.  If not, see <http://www.gnu.org/licenses/>.
 */

package raging.goblin.crypt;

public class Crypt {

    public static void main(String[] args) throws CryptException {
        if (args.length == 2) {
            try {
                System.out.println(crypt(args[0], args[1]));
            } catch (CryptException e) {
                System.err.println(e.getMessage());
            }
        } else {
            printUsage();
        }
    }

    public static String crypt(String toCrypt, String salt) throws CryptException {
        Algorithm algorithm = Algorithm.getAlgorithm(salt);
        if (algorithm == Algorithm.UNKNOWN) {
            throw new CryptException("Could not determine algorithm through provided salt: " + salt);
        }

        return algorithm.crypt(toCrypt, salt);
    }

    private static void printUsage() {
        System.out.println("Crypt - Provides password encryption simular to PHP crypt function.");
        System.out.println("Usage: java -jar Crypt.jar <String to encrypt> <salt>");
        System.out.println("Eg: java -jar Crypt.jar rasmuslerdorf $6$rounds=5000$usesomesillystringforsalt$");
        System.out
            .println("Will generate the output: $6$rounds=5000$usesomesillystri$D4IrlXatmP7rx3P3InaxBeoomnAihCKRVQP22JZ6EY47Wc6BkroIuUUBOov1i.S5KPgErtP/EN5mcO.ChWQW21");
    }

}
