package raging.goblin.infolog.android.data;

import android.content.Context;
import android.database.Cursor;

import java.util.ArrayList;
import java.util.List;

import raging.goblin.infolog.sync.InfologStorage;
import raging.goblin.infolog.sync.data.Infolog;

public class DbStorage implements InfologStorage {

	private DbAdapter db;
	private List<Infolog> infologs;

	public DbStorage(Context context) {
		db = new DbAdapter(context);
	}

	public void start() {
		db.open();
		infologs = fetchInfologsFromDb();
	}

	public void stop() {
		writeInfologsToDb();
		db.close();
	}

	@Override
	public boolean authenticate(String username, String password, String encryptionType) {
		return true;
	}

	@Override
	public void updateInfologs(List<Infolog> infologs, String username, String password, String encryptionType) {
		this.infologs.removeAll(infologs);
		this.infologs.addAll(infologs);
	}

	@Override
	public List<Infolog> addInfologs(List<Infolog> infologs, String username, String password, String encryptionType) {
		this.infologs.addAll(infologs);
		return this.infologs;
	}

	@Override
	public void removeInfologs(List<Infolog> infologs, String username, String password, String encryptionType) {
		this.infologs.removeAll(infologs);
	}

	@Override
	public List<Infolog> getInfologs(String username, String password, String encryptionType) {
		return infologs;
	}

	private List<Infolog> fetchInfologsFromDb() {
		List<Infolog> infologs = new ArrayList<Infolog>();
		Cursor cursor = db.fetchAllNotes();
		cursor.moveToFirst();
		while (!cursor.isAfterLast()) {
			int typeColumn = cursor.getColumnIndex(DbAdapter.KEY_TYPE);
			String type = cursor.getString(typeColumn);
			int egroupwareIdColumn = cursor.getColumnIndex(DbAdapter.KEY_EGROUPWARE_ID);
			int egroupwareId = cursor.getInt(egroupwareIdColumn);
			int subjectColumn = cursor.getColumnIndex(DbAdapter.KEY_TITLE);
			String subject = cursor.getString(subjectColumn);
			int descriptionColumn = cursor.getColumnIndex(DbAdapter.KEY_BODY);
			String description = cursor.getString(descriptionColumn);
			int dateModifiedColumn = cursor.getColumnIndex(DbAdapter.KEY_TIMESTAMP);
			long dateModified = cursor.getLong(dateModifiedColumn);
			infologs.add(new Infolog(egroupwareId, dateModified, subject, description, type));
			cursor.moveToNext();
		}
		return infologs;
	}

	private void writeInfologsToDb() {
		db.deleteAllNotes();
		for (Infolog infolog : infologs) {
			db.createNote(infolog.getSubject(), infolog.getDescription(), infolog.getType(), infolog.getDatemodified(),
				 infolog.getId());
		}
	}

}
