package raging.goblin.infolog.android.gui;

import raging.goblin.infolog.android.data.DbAdapter;

import android.content.Intent;
import android.database.Cursor;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageButton;
import android.widget.SimpleCursorAdapter;
import android.widget.TextView;

public class ColoredCursorAdapter extends SimpleCursorAdapter {

	private InfologsList infologsList;
	private LayoutInflater inflater;

	public ColoredCursorAdapter(InfologsList infologsList, int layout, Cursor c, String[] from, int[] to) {
		super(infologsList, layout, c, from, to);
		this.infologsList = infologsList;
		inflater = LayoutInflater.from(infologsList);
	}

	@Override
	public View getView(final int position, View convertView, ViewGroup parent) {
		Cursor cursor = (Cursor) getItem(position);
		int typeColumn = cursor.getColumnIndex(DbAdapter.KEY_TYPE);
		final String type = cursor.getString(typeColumn);
		int idColumn = cursor.getColumnIndex(DbAdapter.KEY_ROWID);
		final long id = cursor.getLong(idColumn);
		int egroupwareIdColumn = cursor.getColumnIndex(DbAdapter.KEY_EGROUPWARE_ID);
		final long egroupwareId = cursor.getLong(egroupwareIdColumn);

		if (convertView == null) {
			convertView = inflater.inflate(R.layout.infolog_row, null);
		}

		ImageButton deleteButton = (ImageButton) convertView.findViewById(R.id.buttonDeleteInList);
		deleteButton.setOnClickListener(new DeleteClickListener(id));

		StickyNote stickyNote = (StickyNote) convertView.findViewById(R.id.stickyNoteInListBackground);
		stickyNote.setColor(InfologColors.getColor(type));
		stickyNote.setOnClickListener(new EditClickListener(id, type, egroupwareId));

		TextView title = (TextView) convertView.findViewById(R.id.textInfologTitleInList);
		title.setText(cursor.getString(cursor.getColumnIndex(DbAdapter.KEY_TITLE)));

		DateTimeTextView dateTime = (DateTimeTextView) convertView.findViewById(R.id.textInfologTimestamp);
		dateTime.setText(cursor.getString(cursor.getColumnIndex(DbAdapter.KEY_TIMESTAMP)));

		return stickyNote;
	}

	private class EditClickListener implements View.OnClickListener {

		private long id;
		private String type;
		private long egroupwareId;

		public EditClickListener(long id, String type, long egroupwareId) {
			this.id = id;
			this.type = type;
			this.egroupwareId = egroupwareId;
		}

		public void onClick(View view) {
			Intent intent = new Intent(infologsList, InfologEdit.class);
			intent.putExtra(DbAdapter.KEY_ROWID, id);
			intent.putExtra(DbAdapter.KEY_TYPE, type);
			intent.putExtra(DbAdapter.KEY_EGROUPWARE_ID, egroupwareId);
			intent.putExtra(InfologEdit.KEY_MODE, InfologEdit.MODE.VIEW);
			infologsList.startActivity(intent);
		}
	}

	private class DeleteClickListener implements View.OnClickListener {

		private long id;

		public DeleteClickListener(long id) {
			this.id = id;
		}

		public void onClick(View v) {
			DbAdapter db = new DbAdapter(infologsList);
			db.open();
			db.deleteNote(id);
			db.close();
			infologsList.fillData();
		}
	}
}
