package raging.goblin.infolog.android.gui;

import android.content.SharedPreferences;
import android.content.SharedPreferences.OnSharedPreferenceChangeListener;
import android.os.Bundle;
import android.preference.EditTextPreference;
import android.preference.ListPreference;
import android.preference.Preference;
import android.preference.PreferenceActivity;
import android.preference.PreferenceManager;
import android.preference.PreferenceScreen;

/**
 * Preferences activities should extend this class. It sets te current value for a preference in the description
 * field. Override {@link #addPreferences()} method with stuff normally done in the {@link #onCreate(Bundle)} method.
 */
public abstract class CurrentValuePreferencesActivity extends PreferenceActivity implements
	 OnSharedPreferenceChangeListener {

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		addPreferences();
		setCurrentValues(getPreferenceScreen());
	}

	@Override
	public void onSharedPreferenceChanged(SharedPreferences arg0, String arg1) {
		setCurrentValues(getPreferenceScreen());
	}

	@Override
	protected void onResume() {
		super.onResume();
		getPreferenceScreen().getSharedPreferences().registerOnSharedPreferenceChangeListener(this);
	}

	@Override
	protected void onPause() {
		super.onPause();
		getPreferenceScreen().getSharedPreferences().unregisterOnSharedPreferenceChangeListener(this);
	}

	/**
	 * Adds the preferences from xml or dynamically here, normally this is done in the {@link #onCreate(Bundle)} method.
	 * Example usage:<br />
	 * <code>
	 * &nbsp; protected void addPreferences() {<br />
	 * &nbsp;&nbsp;&nbsp; addPreferencesFromResource(R.xml.connectionpreferences);</br />
	 * &nbsp; }
	 * </code>
	 */
	abstract protected void addPreferences();

	private void setCurrentValues(PreferenceScreen screen) {
		SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(this);
		for (int i = 0; i < screen.getPreferenceCount(); i++) {
			Preference preference = screen.getPreference(i);
			if (preference instanceof ListPreference) {
				String value = preferences.getString(preference.getKey(), ((ListPreference) preference).getEntry().toString());
				preference.setSummary(value);
			} else if (preference instanceof EditPasswordPreference) {
				// Nothing to do
			} else if (preference instanceof EditTextPreference) {
				String value = preferences.getString(preference.getKey(), ((EditTextPreference) preference).getText());
				preference.setSummary(value);
			}
		}
	}

}