package raging.goblin.infolog.android.gui;

import android.content.Context;
import android.preference.EditTextPreference;
import android.util.AttributeSet;
import android.util.Log;

import org.jasypt.exceptions.EncryptionOperationNotPossibleException;

import raging.goblin.infolog.android.util.JasyptCrypt;

public class EditPasswordPreference extends EditTextPreference {

	private static final String SHOW_VALUE = "**********";
	private static final String LOG_TAG = "EditPasswordPreference";

	public EditPasswordPreference(Context context, AttributeSet attrs, int defStyle) {
		super(context, attrs, defStyle);
	}

	public EditPasswordPreference(Context context, AttributeSet attrs) {
		super(context, attrs);
	}

	public EditPasswordPreference(Context context) {
		super(context);
	}

	@Override
	public CharSequence getSummary() {
		return SHOW_VALUE;
	}

	@Override
	public void setText(String text) {
		if (!JasyptCrypt.isEncrypted(text)) {
			text = JasyptCrypt.encryptPassword(text.toCharArray());
		}
		super.setText(text);
	}

	@Override
	public String getText() {
		String text = super.getText();
		if (JasyptCrypt.isEncrypted(text)) {
			try {
				return JasyptCrypt.decryptPassword(text);
			} catch (EncryptionOperationNotPossibleException e) {
				Log.e(LOG_TAG, "Unable to decrypt password", e);
			}
		}
		return super.getText();
	}

}