package raging.goblin.infolog.android.gui;

import android.app.Activity;
import android.content.Context;
import android.database.Cursor;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;

import java.util.Date;

import raging.goblin.infolog.android.data.DbAdapter;

public class InfologEdit extends Activity {

	public enum MODE {VIEW, EDIT, CREATE}

	;

	public static final String KEY_MODE = "KEY_MODE";
	private static final String LOG_TAG = "InfologEdit";

	private EditText titleEditText;
	private EditText bodyEditText;
	private DateTimeTextView timestampTextView;
	private StickyNote stickyNote;
	private Button editButton;
	private Button deleteButton;
	private Button saveButton;
	private Button cancelButton;

	private MODE mode;
	private Long rowId;
	private DbAdapter db;
	private String type;
	private long egroupwareId;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		type = (String) getIntent().getExtras().get(DbAdapter.KEY_TYPE);
		egroupwareId = getIntent().getExtras().containsKey(DbAdapter.KEY_EGROUPWARE_ID) ? getIntent().getExtras()
			 .getLong(DbAdapter.KEY_EGROUPWARE_ID) : -1;
		db = new DbAdapter(this);
		db.open();
		setContentView(R.layout.infolog_edit);

		titleEditText = (EditText) findViewById(R.id.editTextTitle);
		bodyEditText = (EditText) findViewById(R.id.editTextBody);
		timestampTextView = (DateTimeTextView) findViewById(R.id.textViewTimestamp);
		stickyNote = (StickyNote) findViewById(R.id.stickyNoteBackground);
		editButton = (Button) findViewById(R.id.buttonEdit);
		deleteButton = (Button) findViewById(R.id.buttonDelete);
		saveButton = (Button) findViewById(R.id.buttonSave);
		cancelButton = (Button) findViewById(R.id.buttonCancel);

		rowId = (savedInstanceState == null) ? null : (Long) savedInstanceState.getSerializable(DbAdapter.KEY_ROWID);
		if (rowId == null && getIntent().getExtras().containsKey(DbAdapter.KEY_ROWID)) {
			rowId = getIntent().getExtras().getLong(DbAdapter.KEY_ROWID);
		}

		stickyNote.setColor(InfologColors.getColor(type));
		populateFields();
		setMode((MODE) getIntent().getExtras().get(InfologEdit.KEY_MODE));
	}

	@Override
	protected void onResume() {
		super.onResume();
		populateFields();
	}

	@Override
	protected void onDestroy() {
		super.onDestroy();
		db.close();
	}

	public void edit(View view) {
		setMode(MODE.EDIT);
	}

	public void delete(View view) {
		db.deleteNote(rowId);
		finish();
	}

	public void save(View view) {
		saveToDb();
		populateFields();
		setMode(MODE.VIEW);
	}

	public void cancel(View view) {
		if (mode == MODE.CREATE) {
			finish();
		} else if (mode == MODE.EDIT) {
			populateFields();
			setMode(MODE.VIEW);
		}
	}

	private void setMode(MODE mode) {
		this.mode = mode;
		updateEditTextsToMode();
		updateButtonsToMode();
		updateKeyboardToMode();
	}

	private void updateKeyboardToMode() {
		final InputMethodManager inputMethodManager = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
		switch (mode) {
			case VIEW:
				inputMethodManager.hideSoftInputFromInputMethod(bodyEditText.getWindowToken(), 0);
				break;
			case EDIT:
				inputMethodManager.showSoftInput(bodyEditText, InputMethodManager.SHOW_IMPLICIT);
				break;
			case CREATE:
				new Thread() {
					@Override
					public void run() {
						try {
							Thread.sleep(250);
						} catch (InterruptedException e) {
							Log.e(LOG_TAG, "Interrupted sleep", e);
						}
						runOnUiThread(new Runnable() {
							@Override
							public void run() {
								inputMethodManager.showSoftInput(titleEditText, InputMethodManager.SHOW_IMPLICIT);
							}
						});
					}
				}.start();
				break;
		}
	}

	private void updateEditTextsToMode() {
		titleEditText.setEnabled(mode != MODE.VIEW);
		bodyEditText.setEnabled(mode != MODE.VIEW);

		if (mode == MODE.EDIT) {
			bodyEditText.requestFocus();
			bodyEditText.setSelection(bodyEditText.getText().length());
		} else if (mode == MODE.CREATE) {
			titleEditText.requestFocus();
			titleEditText.setSelection(titleEditText.getText().length());
		}
	}

	private void updateButtonsToMode() {
		editButton.setVisibility(mode == MODE.VIEW ? View.VISIBLE : View.GONE);
		deleteButton.setVisibility(mode == MODE.VIEW ? View.VISIBLE : View.GONE);
		saveButton.setVisibility(mode == MODE.VIEW ? View.GONE : View.VISIBLE);
		cancelButton.setVisibility(mode == MODE.VIEW ? View.GONE : View.VISIBLE);
		if (mode == MODE.VIEW) {
			editButton.requestFocus();
		}
	}

	private void populateFields() {
		if (rowId != null) {
			Cursor note = db.fetchNote(rowId);
			startManagingCursor(note);
			titleEditText.setText(note.getString(note.getColumnIndex(DbAdapter.KEY_TITLE)));
			bodyEditText.setText(note.getString(note.getColumnIndex(DbAdapter.KEY_BODY)));
			timestampTextView.setText(note.getString(note.getColumnIndex(DbAdapter.KEY_TIMESTAMP)));
		}
	}

	private void saveToDb() {
		String title = titleEditText.getText().toString();
		String body = bodyEditText.getText().toString();
		long modificationDate = new Date().getTime();

		if (rowId == null) {
			long id = db.createNote(title, body, type.toString(), modificationDate, egroupwareId);
			if (id > 0) {
				rowId = id;
			}
		} else {
			db.updateNote(rowId, title, body, type, modificationDate, egroupwareId);
		}
	}
}
