package raging.goblin.infolog.android.gui;

import raging.goblin.infolog.android.data.DbAdapter;
import raging.goblin.infolog.android.sync.SynchronizeActivity;
import raging.goblin.infolog.sync.data.Infolog;

import android.app.AlertDialog;
import android.app.ListActivity;
import android.content.Intent;
import android.database.Cursor;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.TextView;

public class InfologsList extends ListActivity {

	private DbAdapter db;
	private ColoredCursorAdapter notes;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.infologs_list);
		db = new DbAdapter(this);
		db.open();
		fillData();
	}

	@Override
	protected void onDestroy() {
		super.onDestroy();
		db.close();
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.infologs_list_menu, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
			case R.id.about:
				showAbout();
				return true;
			case R.id.synchronize:
				startSynchronizeActivity();
				return true;
			default:
				return super.onOptionsItemSelected(item);
		}
	}

	public void createNote(View view) {
		createInfolog(Infolog.NOTE);
	}

	public void createPhoneCall(View view) {
		createInfolog(Infolog.PHONE);
	}

	public void createTask(View view) {
		createInfolog(Infolog.TASK);
	}

	public void fillData() {
		Cursor notesCursor = db.fetchAllNotes();
		startManagingCursor(notesCursor);
		String[] from = new String[]{DbAdapter.KEY_TITLE, DbAdapter.KEY_TIMESTAMP};
		int[] to = new int[]{R.id.textInfologTitleInList, R.id.textInfologTimestamp};
		notes = new ColoredCursorAdapter(this, R.layout.infolog_row, notesCursor, from, to);
		setListAdapter(notes);
	}

	private void createInfolog(String type) {
		Intent intent = new Intent(this, InfologEdit.class);
		intent.putExtra(DbAdapter.KEY_TYPE, type);
		intent.putExtra(InfologEdit.KEY_MODE, InfologEdit.MODE.CREATE);
		startActivity(intent);
	}

	private void startSynchronizeActivity() {
		Intent intent = new Intent(this, SynchronizeActivity.class);
		startActivity(intent);
	}

	protected void showAbout() {
		View messageView = getLayoutInflater().inflate(R.layout.about, null, false);
		TextView textView = (TextView) messageView.findViewById(R.id.about_credits);
		int defaultColor = textView.getTextColors().getDefaultColor();
		textView.setTextColor(defaultColor);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setIcon(R.drawable.icon);
		builder.setTitle(R.string.app_name);
		builder.setView(messageView);
		builder.create();
		builder.show();
	}

}
