/*
 * Copyright 2013, RagingGoblin <http://raginggoblin.wordpress.com>
 * 
 * This file is part of Infolog.
 *
 *  Infolog is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Infolog is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Infolog.  If not, see <http://www.gnu.org/licenses/>.
 */

package raging.goblin.infolog.client;

import javax.swing.UIManager;
import javax.swing.UIManager.LookAndFeelInfo;
import javax.swing.UnsupportedLookAndFeelException;

import org.apache.log4j.Logger;

import raging.goblin.infolog.client.gui.ClientGui;
import raging.goblin.infolog.client.properties.PropertyLoader;

/**
 * The ClientApplication is the main starting point to get a client running. It is controlled by the user through the
 * ClientGui. It implements the Singleton design pattern.
 */
public class ClientApplication {

	private static Logger logger = Logger.getLogger("raging.goblin.infolog.client.ClientApplication");
	private Client client;

	private ClientApplication() {
		loadLaf();
		Runtime.getRuntime().addShutdownHook(new ShutdownHook());
		new PropertyLoader().loadProperties();
		JSONStorage jsonStorage = new JSONStorage();
		client = new Client(jsonStorage);
		client.startSynchronizing();
		new ClientGui(client);
	}

	public static void main(String[] args) {
		System.setProperty("awt.useSystemAAFontSettings", "on");
		System.setProperty("swing.aatext", "true");

		new ClientApplication();
	}

	private void loadLaf() {
		try {
			if (UIManager.getSystemLookAndFeelClassName().contains("Metal")) {
				setLinuxLaf();
			} else {
				UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
				logger.debug("Setting system look and feel");
			}
		} catch (Exception e) {
			logger.error("Could not set system look and feel");
			logger.debug(e.getMessage(), e);
		}
	}

	private void setLinuxLaf() throws ClassNotFoundException, InstantiationException, IllegalAccessException,
	   UnsupportedLookAndFeelException {
		try {
			UIManager.setLookAndFeel("com.sun.java.swing.plaf.gtk.GTKLookAndFeel");
			logger.debug("Setting GTK look and feel");
		} catch (Exception e) {
			setNimbusLaf();
		}
	}

	private void setNimbusLaf() throws ClassNotFoundException, InstantiationException, IllegalAccessException,
	   UnsupportedLookAndFeelException {
		for (LookAndFeelInfo info : UIManager.getInstalledLookAndFeels()) {
			if ("Nimbus".equals(info.getName())) {
				UIManager.setLookAndFeel(info.getClassName());
				logger.debug("Setting Nimbus look and feel");
				break;
			}
		}
	}

	/**
	 * This class provides a shutdownhook according to:
	 * 
	 * <a href="http://onjava.com/pub/a/onjava/2003/03/26/shutdownhook.html">
	 * http://onjava.com/pub/a/onjava/2003/03/26/shutdownhook.html</a>
	 */
	private class ShutdownHook extends Thread {
		@Override
		public void run() {
			client.stopSynchronizing();
			client.synchronize();
		}
	}
}
