/*
 * Copyright 2013, RagingGoblin <http://raginggoblin.wordpress.com>
 * 
 * This file is part of Infolog.
 *
 *  Infolog is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Infolog is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Infolog.  If not, see <http://www.gnu.org/licenses/>.
 */

package raging.goblin.infolog.client.gui;

import java.awt.Color;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.border.Border;

/**
 * A label displaying a certain color, by clicking on the user will be capable of changing the color.
 */
public class ColorButton extends JLabel implements MouseListener {

	private int[] color;
	private ColorChooser chooser;

	public ColorButton(int[] color) {
		this.color = color;
		this.chooser = new ColorChooser(getColor());
		init();
	}

	public String getPropertiesValue() {
		return "" + color[0] + "," + color[1] + "," + color[2];
	}

	public int[] getColorValue() {
		return color;
	}

	public void setColor(int[] color) {
		this.color = color;
		chooser.setColor(getColor());
		setBackground(getColor());
	}

	private void init() {
		setOpaque(true);
		setBackground(getColor());
		Border border = BorderFactory.createRaisedBevelBorder();
		setBorder(border);
		addMouseListener(this);
	}

	@Override
	public void mouseClicked(MouseEvent e) {
		// Nothing to do
	}

	@Override
	public void mousePressed(MouseEvent e) {
		requestFocusInWindow();
		chooser.setColor(getColor());
		chooser.setVisible(true);
		if (chooser.isOkButtonPressed()) {
			color = getColorValues(chooser.getColor());
			setBackground(getColor());
		}
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		// Nothing to do
	}

	@Override
	public void mouseEntered(MouseEvent e) {
		// Nothing to do
	}

	@Override
	public void mouseExited(MouseEvent e) {
		// Nothing to do
	}

	private Color getColor() {
		return new Color(color[0], color[1], color[2]);
	}

	private int[] getColorValues(Color awtColor) {
		int red = awtColor.getRed();
		int green = awtColor.getGreen();
		int blue = awtColor.getBlue();
		return new int[] { red, green, blue };
	}
}
