/*
 * Copyright 2013, RagingGoblin <http://raginggoblin.wordpress.com>
 * 
 * This file is part of Infolog.
 *
 *  Infolog is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Infolog is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Infolog.  If not, see <http://www.gnu.org/licenses/>.
 */

package raging.goblin.infolog.client.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dialog;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.jgoodies.forms.factories.FormFactory;
import com.jgoodies.forms.layout.ColumnSpec;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.forms.layout.RowSpec;

/**
 * A dialog displaying a color chooser to choose a color.
 */
public class ColorChooser extends JDialog implements ChangeListener {

	private boolean okButtonPressed = false;
	private Color color;
	private JColorChooser colorChooser;

	public ColorChooser(Color color) {
		JPanel actionPanel = new JPanel();
		getContentPane().add(actionPanel, BorderLayout.SOUTH);
		actionPanel.setLayout(new FormLayout(new ColumnSpec[] { ColumnSpec.decode("default:grow"),
		   FormFactory.RELATED_GAP_COLSPEC, ColumnSpec.decode("max(40dlu;default)"), FormFactory.RELATED_GAP_COLSPEC,
		   FormFactory.DEFAULT_COLSPEC, FormFactory.RELATED_GAP_COLSPEC, ColumnSpec.decode("max(40dlu;default)"),
		   FormFactory.RELATED_GAP_COLSPEC, ColumnSpec.decode("max(5dlu;default)"), }, new RowSpec[] {
		   FormFactory.DEFAULT_ROWSPEC, FormFactory.RELATED_GAP_ROWSPEC, FormFactory.DEFAULT_ROWSPEC,
		   FormFactory.RELATED_GAP_ROWSPEC, RowSpec.decode("max(5dlu;default)"), }));

		JButton btnCancel = new JButton("Cancel");
		btnCancel.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				okButtonPressed = false;
				setVisible(false);
			}
		});
		actionPanel.add(btnCancel, "3, 3, 2, 1");

		JButton btnOk = new JButton("OK");
		btnOk.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				okButtonPressed = true;
				setVisible(false);
			}
		});
		actionPanel.add(btnOk, "7, 3, 2, 1");

		colorChooser = new JColorChooser(color);
		colorChooser.getSelectionModel().addChangeListener(this);
		getContentPane().add(colorChooser, BorderLayout.CENTER);

		setSize(700, 500);
		setModalityType(Dialog.DEFAULT_MODALITY_TYPE);
	}

	@Override
	public void stateChanged(ChangeEvent e) {
		color = colorChooser.getColor();
	}

	public boolean isOkButtonPressed() {
		return okButtonPressed;
	}

	public Color getColor() {
		return color;
	}

	public void setColor(Color color) {
		this.color = color;
		colorChooser.setColor(color);
	}
}
