/*
 * Copyright 2013, RagingGoblin <http://raginggoblin.wordpress.com>
 * 
 * This file is part of Infolog.
 *
 *  Infolog is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Infolog is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Infolog.  If not, see <http://www.gnu.org/licenses/>.
 */

package raging.goblin.infolog.client.gui;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

import raging.goblin.infolog.client.properties.InfologClientProperties;
import raging.goblin.infolog.client.properties.Properties;

import com.jgoodies.forms.factories.FormFactory;
import com.jgoodies.forms.layout.ColumnSpec;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.forms.layout.RowSpec;

/**
 * A dialog to configure the look and feel of the gui and notes.
 */
public class GuiConfigDialog extends JDialog {

	private static final String TITLE = "Configure look and feel";
	private static final String WIDTH_TOOLTIP = "Width of a sticky note in px.";
	private static final String HEIGHT_TOOLTIP = "Height of a sticky note in px.";
	private static final String SYSTRAY_TOOLTIP = "Path to icon used in system tray (requires restart).";
	private static final String SPACING_TOOLTIP = "Offset in px. between sticky notes when layed out on the screen.";
	private static final String NOTE_BOTTOM_TOOLTIP = "Bottom color used for sticky notes of type 'note'.";
	private static final String NOTE_TOP_TOOLTIP = "Top color used for sticky notes of type 'note'.";
	private static final String PHONE_BOTTOM_TOOLTIP = "Bottom color used for sticky notes of type 'phone call'.";
	private static final String PHONE_TOP_TOOLTIP = "Top color used for sticky notes of type 'phone call'.";
	private static final String TASK_BOTTOM_TOOLTIP = "Bottom color used for sticky notes of type 'task'.";
	private static final String TASK_TOP_TOOLTIP = "Top color used for sticky notes of type 'task'.";
	private static final String DRAG_DELAY_TOOLTIP = "Threshold value to tweak the drag behaviour, higher values will delay the repaint events making it easier for slow graphic cards to keep up.";
	private static final String DOUBLE_CLICK_DELAY_TOOLTIP = "Delay in ms between two clicks to register as a double click.";

	private JSpinner widthSpinner;
	private JSpinner spacingSpinner;
	private JSpinner heightSpinner;
	private ColorButton btnTopColorNotes;
	private ColorButton btnTopColorTasks;
	private ColorButton btnTopColorPhoneCalls;
	private ColorButton btnBottomColorNotes;
	private ColorButton btnBottomColorTasks;
	private ColorButton btnBottomColorPhoneCalls;
	private JSpinner dragDelaySpinner;
	private JSpinner doubleClickDelaySpinner;
	private JTextField trayIconField;

	public GuiConfigDialog(JFrame parent) {
		super(parent, TITLE, true);
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		setSize(850, 550);

		JPanel actionPanel = new JPanel();
		getContentPane().add(actionPanel, BorderLayout.SOUTH);
		actionPanel.setLayout(new FormLayout(new ColumnSpec[] { ColumnSpec.decode("default:grow"),
		   FormFactory.RELATED_GAP_COLSPEC, FormFactory.DEFAULT_COLSPEC, FormFactory.RELATED_GAP_COLSPEC,
		   FormFactory.DEFAULT_COLSPEC, FormFactory.RELATED_GAP_COLSPEC, ColumnSpec.decode("max(40dlu;default)"),
		   FormFactory.RELATED_GAP_COLSPEC, FormFactory.DEFAULT_COLSPEC, FormFactory.RELATED_GAP_COLSPEC,
		   ColumnSpec.decode("max(40dlu;default)"), FormFactory.RELATED_GAP_COLSPEC,
		   ColumnSpec.decode("max(5dlu;default)"), }, new RowSpec[] { FormFactory.DEFAULT_ROWSPEC,
		   FormFactory.RELATED_GAP_ROWSPEC, RowSpec.decode("max(5dlu;default)"), }));

		JButton btnCancel = new JButton("Cancel");
		btnCancel.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				setVisible(false);
			}
		});
		actionPanel.add(btnCancel, "7, 1, 2, 1");

		JButton btnDefaults = new JButton("Load defaults");
		btnDefaults.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				setDefaults();
			}
		});
		actionPanel.add(btnDefaults, "3, 1");

		JButton btnOk = new JButton("OK");
		btnOk.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				setVisible(false);
				saveConfiguration();
			}
		});
		actionPanel.add(btnOk, "11, 1, 2, 1");

		JPanel configPanel = new JPanel();
		getContentPane().add(configPanel, BorderLayout.CENTER);
		configPanel.setLayout(new FormLayout(new ColumnSpec[] { ColumnSpec.decode("max(61dlu;min):grow"),
		   ColumnSpec.decode("max(40dlu;default):grow"), FormFactory.RELATED_GAP_COLSPEC, ColumnSpec.decode("99px:grow"),
		   FormFactory.RELATED_GAP_COLSPEC, ColumnSpec.decode("max(30dlu;default)"), FormFactory.RELATED_GAP_COLSPEC,
		   FormFactory.DEFAULT_COLSPEC, FormFactory.RELATED_GAP_COLSPEC, ColumnSpec.decode("default:grow"),
		   FormFactory.RELATED_GAP_COLSPEC, FormFactory.DEFAULT_COLSPEC, FormFactory.RELATED_GAP_COLSPEC,
		   ColumnSpec.decode("default:grow"), FormFactory.RELATED_GAP_COLSPEC, ColumnSpec.decode("max(30dlu;default)"),
		   FormFactory.RELATED_GAP_COLSPEC, ColumnSpec.decode("max(30dlu;default)"), }, new RowSpec[] {
		   FormFactory.RELATED_GAP_ROWSPEC, FormFactory.DEFAULT_ROWSPEC, FormFactory.RELATED_GAP_ROWSPEC,
		   FormFactory.DEFAULT_ROWSPEC, FormFactory.DEFAULT_ROWSPEC, FormFactory.DEFAULT_ROWSPEC,
		   FormFactory.RELATED_GAP_ROWSPEC, FormFactory.DEFAULT_ROWSPEC, FormFactory.RELATED_GAP_ROWSPEC,
		   FormFactory.DEFAULT_ROWSPEC, FormFactory.RELATED_GAP_ROWSPEC, FormFactory.DEFAULT_ROWSPEC,
		   FormFactory.RELATED_GAP_ROWSPEC, FormFactory.DEFAULT_ROWSPEC, FormFactory.RELATED_GAP_ROWSPEC,
		   FormFactory.DEFAULT_ROWSPEC, FormFactory.RELATED_GAP_ROWSPEC, FormFactory.DEFAULT_ROWSPEC,
		   FormFactory.RELATED_GAP_ROWSPEC, FormFactory.DEFAULT_ROWSPEC, FormFactory.RELATED_GAP_ROWSPEC,
		   FormFactory.DEFAULT_ROWSPEC, }));

		StringSeparator dimensionsSeparator = new StringSeparator("Dimensions");
		configPanel.add(dimensionsSeparator, "1, 2, 18, 1");

		JLabel lblWidth = new JLabel("Width");
		configPanel.add(lblWidth, "4, 4, right, center");

		widthSpinner = new JSpinner();
		widthSpinner.setToolTipText(WIDTH_TOOLTIP);
		configPanel.add(widthSpinner, "6, 4");

		JLabel lblPx_1 = new JLabel("px");
		configPanel.add(lblPx_1, "8, 4");

		JLabel lblHeight = new JLabel("Height");
		configPanel.add(lblHeight, "12, 4, 3, 1, right, center");

		heightSpinner = new JSpinner();
		heightSpinner.setToolTipText(HEIGHT_TOOLTIP);
		configPanel.add(heightSpinner, "16, 4");

		JLabel lblPx = new JLabel("px");
		configPanel.add(lblPx, "18, 4");

		JLabel lblSpacing = new JLabel("Spacing:");
		configPanel.add(lblSpacing, "4, 5, right, center");

		spacingSpinner = new JSpinner();
		spacingSpinner.setToolTipText(SPACING_TOOLTIP);
		configPanel.add(spacingSpinner, "6, 5");

		JLabel lblPx_2 = new JLabel("px");
		configPanel.add(lblPx_2, "8, 5");

		StringSeparator colorsSeparator = new StringSeparator("Colors");
		configPanel.add(colorsSeparator, "1, 6, 18, 1");

		JLabel lblNotes = new JLabel("Notes");
		configPanel.add(lblNotes, "2, 8, right, default");

		JLabel lblTop = new JLabel("Top:");
		configPanel.add(lblTop, "4, 8, right, center");

		btnTopColorNotes = new ColorButton(InfologClientProperties.noteTopColor);
		btnTopColorNotes.setToolTipText(NOTE_TOP_TOOLTIP);
		configPanel.add(btnTopColorNotes, "6, 8, fill, fill");

		JLabel lblBottom = new JLabel("Bottom:");
		configPanel.add(lblBottom, "12, 8, 3, 1, right, center");

		btnBottomColorNotes = new ColorButton(InfologClientProperties.noteBottomColor);
		btnBottomColorNotes.setToolTipText(NOTE_BOTTOM_TOOLTIP);
		configPanel.add(btnBottomColorNotes, "16, 8, fill, fill");

		JLabel lblTasks = new JLabel("Tasks");
		configPanel.add(lblTasks, "2, 10, right, default");

		JLabel lblTop_1 = new JLabel("Top:");
		configPanel.add(lblTop_1, "4, 10, right, center");

		btnTopColorTasks = new ColorButton(InfologClientProperties.taskTopColor);
		btnTopColorTasks.setToolTipText(TASK_TOP_TOOLTIP);
		configPanel.add(btnTopColorTasks, "6, 10, fill, fill");

		JLabel lblBottom_1 = new JLabel("Bottom:");
		configPanel.add(lblBottom_1, "12, 10, 3, 1, right, center");

		btnBottomColorTasks = new ColorButton(InfologClientProperties.taskBottomColor);
		btnBottomColorTasks.setToolTipText(TASK_BOTTOM_TOOLTIP);
		configPanel.add(btnBottomColorTasks, "16, 10, fill, fill");

		JLabel lblPhoneCalls = new JLabel("Phone Calls");
		configPanel.add(lblPhoneCalls, "2, 12, right, default");

		JLabel lblTop_2 = new JLabel("Top:");
		configPanel.add(lblTop_2, "4, 12, right, center");

		btnTopColorPhoneCalls = new ColorButton(InfologClientProperties.phoneTopColor);
		btnTopColorPhoneCalls.setToolTipText(PHONE_TOP_TOOLTIP);
		configPanel.add(btnTopColorPhoneCalls, "6, 12, fill, fill");

		JLabel lblBottom_2 = new JLabel("Bottom:");
		configPanel.add(lblBottom_2, "12, 12, 3, 1, right, center");

		btnBottomColorPhoneCalls = new ColorButton(InfologClientProperties.phoneBottomColor);
		btnBottomColorPhoneCalls.setToolTipText(PHONE_BOTTOM_TOOLTIP);
		configPanel.add(btnBottomColorPhoneCalls, "16, 12, fill, fill");

		StringSeparator behaviourSeparator = new StringSeparator("Mouse behaviour");
		configPanel.add(behaviourSeparator, "1, 14, 18, 1, fill, default");

		JLabel lblDragDelay = new JLabel("Drag delay: ");
		configPanel.add(lblDragDelay, "4, 16, right, center");

		dragDelaySpinner = new JSpinner();
		dragDelaySpinner.setToolTipText(DRAG_DELAY_TOOLTIP);
		configPanel.add(dragDelaySpinner, "6, 16");

		JLabel lblDoubleClickDelay = new JLabel("Double click delay:");
		configPanel.add(lblDoubleClickDelay, "10, 16, 5, 1, right, center");

		doubleClickDelaySpinner = new JSpinner();
		doubleClickDelaySpinner.setToolTipText(DOUBLE_CLICK_DELAY_TOOLTIP);
		configPanel.add(doubleClickDelaySpinner, "16, 16");

		StringSeparator iconSeparator = new StringSeparator("System tray icon");
		configPanel.add(iconSeparator, "1, 18, 18, 1");

		trayIconField = new JTextField();
		trayIconField.setToolTipText(SYSTRAY_TOOLTIP);
		configPanel.add(trayIconField, "2, 20, 11, 1, fill, default");

		JButton btnChooseIcon = new JButton("Choose file");
		btnChooseIcon.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				JFileChooser fileChooser = new JFileChooser();
				int returnVal = fileChooser.showOpenDialog(GuiConfigDialog.this);
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					File file = fileChooser.getSelectedFile();
					trayIconField.setText(file.getPath());
				}
			}
		});
		configPanel.add(btnChooseIcon, "14, 20, 3, 1, right, default");

		loadConfiguration();
		setFocus(btnCancel);
		setVisible(true);
	}

	private void loadConfiguration() {
		widthSpinner.setValue(InfologClientProperties.width);
		heightSpinner.setValue(InfologClientProperties.height);
		spacingSpinner.setValue(InfologClientProperties.spacing);
		dragDelaySpinner.setValue(InfologClientProperties.dragDelay);
		doubleClickDelaySpinner.setValue(InfologClientProperties.doubleClickDelay);
		trayIconField.setText(InfologClientProperties.trayIcon);
	}

	private void saveConfiguration() {
		new Thread() {
			@Override
			public void run() {
				Properties properties = Properties.getInstance();
				properties.setProperty("width", widthSpinner.getValue().toString());
				properties.setProperty("spacing", spacingSpinner.getValue().toString());
				properties.setProperty("height", heightSpinner.getValue().toString());

				properties.setProperty("notetopcolor", btnTopColorNotes.getPropertiesValue());
				properties.setProperty("notebottomcolor", btnBottomColorNotes.getPropertiesValue());

				properties.setProperty("tasktopcolor", btnTopColorTasks.getPropertiesValue());
				properties.setProperty("taskbottomcolor", btnBottomColorTasks.getPropertiesValue());

				properties.setProperty("phonetopcolor", btnTopColorPhoneCalls.getPropertiesValue());
				properties.setProperty("phonebottomcolor", btnBottomColorPhoneCalls.getPropertiesValue());

				properties.setProperty("dragdelay", dragDelaySpinner.getValue().toString());
				properties.setProperty("doubleclickdelay", doubleClickDelaySpinner.getValue().toString());

				properties.setProperty("systrayicon", trayIconField.getText());

				properties.save();
				applyConfiguration();
			}

		}.start();
	}

	private void applyConfiguration() {
		InfologClientProperties.width = (Integer) widthSpinner.getValue();
		InfologClientProperties.spacing = (Integer) spacingSpinner.getValue();
		InfologClientProperties.height = (Integer) heightSpinner.getValue();

		InfologClientProperties.noteTopColor = btnTopColorNotes.getColorValue();
		InfologClientProperties.noteBottomColor = btnBottomColorNotes.getColorValue();

		InfologClientProperties.taskTopColor = btnTopColorTasks.getColorValue();
		InfologClientProperties.taskBottomColor = btnBottomColorTasks.getColorValue();

		InfologClientProperties.phoneTopColor = btnTopColorPhoneCalls.getColorValue();
		InfologClientProperties.phoneBottomColor = btnBottomColorPhoneCalls.getColorValue();

		InfologClientProperties.dragDelay = (Integer) dragDelaySpinner.getValue();
		InfologClientProperties.doubleClickDelay = (Integer) doubleClickDelaySpinner.getValue();

		InfologClientProperties.trayIcon = trayIconField.getText();

		StickyNoteIcon.colorsRefreshed();
	}

	private void setDefaults() {
		widthSpinner.setValue(InfologClientProperties.DEFAULT_WIDTH);
		spacingSpinner.setValue(InfologClientProperties.DEFAULT_SPACING);
		heightSpinner.setValue(InfologClientProperties.DEFAULT_HEIGHT);
		btnTopColorNotes.setColor(InfologClientProperties.DEFAULT_NOTE_TOP_COLOR);
		btnBottomColorNotes.setColor(InfologClientProperties.DEFAULT_NOTE_BOTTOM_COLOR);
		btnTopColorTasks.setColor(InfologClientProperties.DEFAULT_TASK_TOP_COLOR);
		btnBottomColorTasks.setColor(InfologClientProperties.DEFAULT_TASK_BOTTOM_COLOR);
		btnTopColorPhoneCalls.setColor(InfologClientProperties.DEFAULT_PHONE_TOP_COLOR);
		btnBottomColorPhoneCalls.setColor(InfologClientProperties.DEFAULT_PHONE_BOTTOM_COLOR);
		dragDelaySpinner.setValue(InfologClientProperties.DEFAULT_DRAG_DELAY);
		doubleClickDelaySpinner.setValue(InfologClientProperties.DEFAULT_DOUBLE_CLICK_DELAY);
		trayIconField.setText(InfologClientProperties.DEFAULT_TRAY_ICON);
	}

	private void setFocus(final JComponent component) {
		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				component.requestFocusInWindow();
			}
		});
	}
}
