/*
 * Copyright 2013, RagingGoblin <http://raginggoblin.wordpress.com>
 * 
 * This file is part of Infolog.
 *
 *  Infolog is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Infolog is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Infolog.  If not, see <http://www.gnu.org/licenses/>.
 */

package raging.goblin.infolog.client.gui;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;

import raging.goblin.infolog.client.properties.InfologClientProperties;

/**
 * The StickyNoteIcon creates BufferedImages to use as icons for the StickyNotes on the task bar of your OS.
 */
public abstract class StickyNoteIcon {

	private static BufferedImage noteIcon;
	private static BufferedImage phoneIcon;
	private static BufferedImage taskIcon;

	public static BufferedImage getNoteIcon() {
		if (noteIcon == null) {
			noteIcon = createIcon(InfologClientProperties.noteTopColor, "Note");
		}
		return noteIcon;
	}

	public static BufferedImage getPhoneIcon() {
		if (phoneIcon == null) {
			phoneIcon = createIcon(InfologClientProperties.phoneTopColor, "Phone call");
		}
		return phoneIcon;
	}

	public static BufferedImage getTaskIcon() {
		if (taskIcon == null) {
			taskIcon = createIcon(InfologClientProperties.taskTopColor, "Task");
		}
		return taskIcon;
	}

	public static void colorsRefreshed() {
		noteIcon = null;
		getNoteIcon();
		taskIcon = null;
		getTaskIcon();
		phoneIcon = null;
		getPhoneIcon();
	}

	private static BufferedImage createIcon(int[] background, String title) {
		BufferedImage image = new BufferedImage(100, 100, BufferedImage.TYPE_INT_ARGB);
		Graphics2D g2 = image.createGraphics();

		g2.setColor(new Color(background[0], background[1], background[2]));
		g2.fillRect(10, 10, 80, 80);

		g2.setColor(new Color(StickyNote.getMinimumColor(background[0]), StickyNote.getMinimumColor(background[1]),
		   StickyNote.getMinimumColor(background[2])));
		g2.fillRect(10, 10, 80, 10);

		g2.setColor(Color.BLACK);
		g2.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 5));
		g2.drawString(title, 15, 19);

		return image;
	}
}
