/*
 * Copyright 2013, RagingGoblin <http://raginggoblin.wordpress.com>
 * 
 * This file is part of Infolog.
 *
 *  Infolog is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Infolog is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Infolog.  If not, see <http://www.gnu.org/licenses/>.
 */

package raging.goblin.infolog.client.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashSet;
import java.util.Set;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JRadioButton;
import javax.swing.JSeparator;
import javax.swing.JSpinner;
import javax.swing.JSpinner.DefaultEditor;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import raging.goblin.crypt.EncryptionType;
import raging.goblin.crypt.JasyptCrypt;
import raging.goblin.infolog.client.Client;
import raging.goblin.infolog.client.properties.InfologClientProperties;
import raging.goblin.infolog.client.properties.Properties;

import com.jgoodies.forms.factories.FormFactory;
import com.jgoodies.forms.layout.ColumnSpec;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.forms.layout.RowSpec;

public class SyncConfigDialog extends JDialog {

	private static final String TITLE = "Configure synchronization";
	private static final int DEFAULT_INTERVAL = 10;
	private static final int DEFAULT_PORT = 8087;
	private static final int DEFAULT_HTTPS_PORT = 443;
	private static final int DISABLED_INTERVAL = -1;
	private static final String ENABLE_TOOLTIP = "Enable or disable synchronization.";
	private static final String HOST_TOOLTIP = "Hostname or ip address where Infolog server runs.";
	private static final String USER_TOOLTIP = "Egroupware username.";
	private static final String PASSWORD_TOOLTIP = "Egroupware password.";
	private static final String INTERVAL_TOOLTIP = "Synchronization interval.";
	private static final String ENCRYPTION_TOOLTIP = "Type of encryption used by Egroupware";
	private static final String CLIENT_WINS_TOOLTIP = "In case of conflicts, value on server will be overwritten";
	private static final String SERVER_WINS_TOOLTIP = "In case of conflicts, value on client will be overwritten";
	private static final String LOGIN_SUCCESS_TITLE = "Login successful";
	private static final String LOGIN_SUCCESS = "You logged in successfully";
	private static final String LOGIN_FAILED_TITLE = "Login was not successful.";
	private static final String LOGIN_FAILED = "You are not able to login with these settings, try different settings.";

	private JTextField hostField;
	private JTextField userField;
	private JPasswordField passwordField;
	private JSpinner intervalSpinner;
	private JSpinner portSpinner;
	private JCheckBox enableBox;
	private Set<JComponent> syncComponents = new HashSet<JComponent>();
	private Client client;
	private JCheckBox httpsCBox;
	private JComboBox<EncryptionType> encryptionTypeBox;
	private JRadioButton rdbtnClientWins;
	private JRadioButton rdbtnServerWins;

	public SyncConfigDialog(JFrame parent, final Client client) {
		super(parent, TITLE, true);
		this.client = client;
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		setSize(new Dimension(400, 400));

		JPanel configPanel = new JPanel();
		getContentPane().add(configPanel, BorderLayout.CENTER);
		configPanel.setLayout(new FormLayout(new ColumnSpec[] { ColumnSpec.decode("max(5dlu;default)"),
		   ColumnSpec.decode("right:default"), ColumnSpec.decode("max(5dlu;default)"), FormFactory.RELATED_GAP_COLSPEC,
		   ColumnSpec.decode("76px:grow"), ColumnSpec.decode("default:grow"), FormFactory.DEFAULT_COLSPEC,
		   FormFactory.RELATED_GAP_COLSPEC, ColumnSpec.decode("max(5dlu;default)"), }, new RowSpec[] {
		   FormFactory.DEFAULT_ROWSPEC, FormFactory.RELATED_GAP_ROWSPEC, RowSpec.decode("28px"),
		   FormFactory.NARROW_LINE_GAP_ROWSPEC, FormFactory.DEFAULT_ROWSPEC, FormFactory.RELATED_GAP_ROWSPEC,
		   RowSpec.decode("28px"), FormFactory.NARROW_LINE_GAP_ROWSPEC, RowSpec.decode("28px"),
		   FormFactory.NARROW_LINE_GAP_ROWSPEC, FormFactory.DEFAULT_ROWSPEC, FormFactory.RELATED_GAP_ROWSPEC,
		   RowSpec.decode("28px"), FormFactory.NARROW_LINE_GAP_ROWSPEC, FormFactory.DEFAULT_ROWSPEC,
		   FormFactory.DEFAULT_ROWSPEC, FormFactory.RELATED_GAP_ROWSPEC, FormFactory.DEFAULT_ROWSPEC,
		   FormFactory.RELATED_GAP_ROWSPEC, FormFactory.DEFAULT_ROWSPEC, FormFactory.RELATED_GAP_ROWSPEC,
		   FormFactory.DEFAULT_ROWSPEC, RowSpec.decode("30px"), FormFactory.RELATED_GAP_ROWSPEC,
		   FormFactory.DEFAULT_ROWSPEC, }));

		JLabel lblEnableS = new JLabel("Enable Synchronization");
		configPanel.add(lblEnableS, "2, 1");

		enableBox = new JCheckBox("", isSynchronizationEnabled());
		enableBox.setToolTipText(ENABLE_TOOLTIP);
		enableBox.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				syncEnbledChanged((JCheckBox) e.getSource());
			}
		});
		configPanel.add(enableBox, "5, 1");

		JLabel lblServerHost = new JLabel("Server host");
		configPanel.add(lblServerHost, "2, 3, right, center");

		hostField = new JTextField();
		hostField.setToolTipText(HOST_TOOLTIP);
		configPanel.add(hostField, "5, 3, 2, 1, fill, center");
		hostField.setColumns(10);
		syncComponents.add(hostField);

		JLabel lblServerPort = new JLabel("Port number");
		configPanel.add(lblServerPort, "2, 5, right, center");

		portSpinner = new JSpinner();
		portSpinner.setModel(new SpinnerNumberModel(new Integer(DEFAULT_PORT), null, null, new Integer(1)));
		portSpinner.setEditor(new JSpinner.NumberEditor(portSpinner, "#"));
		syncComponents.add(portSpinner);
		configPanel.add(portSpinner, "5, 5, fill, default");

		httpsCBox = new JCheckBox("Https");
		httpsCBox.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				if (httpsCBox.isSelected()) {
					portSpinner.setValue(DEFAULT_HTTPS_PORT);
				} else {
					portSpinner.setValue(DEFAULT_PORT);
				}
			}
		});
		httpsCBox.setHorizontalTextPosition(SwingConstants.LEFT);
		configPanel.add(httpsCBox, "6, 5, right, center");

		JLabel lblUsername = new JLabel("Username");
		configPanel.add(lblUsername, "2, 7, right, center");

		userField = new JTextField();
		userField.setToolTipText(USER_TOOLTIP);
		configPanel.add(userField, "5, 7, 2, 1, fill, center");
		userField.setColumns(10);
		syncComponents.add(userField);

		JLabel lblPassword = new JLabel("Password");
		configPanel.add(lblPassword, "2, 9, right, center");

		passwordField = new JPasswordField();
		passwordField.setToolTipText(PASSWORD_TOOLTIP);
		configPanel.add(passwordField, "5, 9, 2, 1, fill, center");
		syncComponents.add(passwordField);

		JLabel lblEncryptionType = new JLabel("Encryption type");
		configPanel.add(lblEncryptionType, "2, 11");

		encryptionTypeBox = new JComboBox<EncryptionType>(EncryptionType.values());
		encryptionTypeBox.setToolTipText(ENCRYPTION_TOOLTIP);
		configPanel.add(encryptionTypeBox, "5, 11, 2, 1, fill, default");
		syncComponents.add(encryptionTypeBox);

		JLabel lblSynchronizationInterval = new JLabel("Interval");
		configPanel.add(lblSynchronizationInterval, "2, 13, right, center");

		intervalSpinner = new JSpinner();
		intervalSpinner.setToolTipText(INTERVAL_TOOLTIP);
		intervalSpinner.setModel(new SpinnerNumberModel(new Integer(10), new Integer(1), null, new Integer(1)));
		((DefaultEditor) intervalSpinner.getEditor()).getTextField().setColumns(4);
		configPanel.add(intervalSpinner, "5, 13, fill, center");
		syncComponents.add(intervalSpinner);

		JLabel lblMin = new JLabel("min");
		configPanel.add(lblMin, "6, 13, left, center");

		JLabel lblConflicts = new JLabel("Conflicts");
		configPanel.add(lblConflicts, "2, 15");

		ButtonGroup group = new ButtonGroup();
		rdbtnClientWins = new JRadioButton("Client wins");
		rdbtnClientWins.setToolTipText(CLIENT_WINS_TOOLTIP);
		configPanel.add(rdbtnClientWins, "5, 15, 2, 1");
		syncComponents.add(rdbtnClientWins);
		group.add(rdbtnClientWins);

		rdbtnServerWins = new JRadioButton("Server wins");
		rdbtnServerWins.setToolTipText(SERVER_WINS_TOOLTIP);
		configPanel.add(rdbtnServerWins, "5, 16, 2, 1");
		syncComponents.add(rdbtnServerWins);
		group.add(rdbtnServerWins);

		JButton btnTestCredentials = new JButton("Test credentials");
		btnTestCredentials.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				tryLogin();
			}
		});

		configPanel.add(btnTestCredentials, "5, 20, 2, 1, fill, center");
		syncComponents.add(btnTestCredentials);

		JSeparator separator = new JSeparator();
		configPanel.add(separator, "2, 23, 5, 1");

		JPanel actionPanel = new JPanel();
		getContentPane().add(actionPanel, BorderLayout.SOUTH);
		actionPanel.setLayout(new FormLayout(new ColumnSpec[] { ColumnSpec.decode("default:grow"),
		   FormFactory.RELATED_GAP_COLSPEC, ColumnSpec.decode("max(40dlu;default)"), FormFactory.RELATED_GAP_COLSPEC,
		   FormFactory.DEFAULT_COLSPEC, FormFactory.RELATED_GAP_COLSPEC, ColumnSpec.decode("max(40dlu;default)"),
		   FormFactory.RELATED_GAP_COLSPEC, ColumnSpec.decode("max(5dlu;default)"), }, new RowSpec[] {
		   FormFactory.DEFAULT_ROWSPEC, FormFactory.RELATED_GAP_ROWSPEC, RowSpec.decode("max(5dlu;default)"), }));

		JButton btnCancel = new JButton("Cancel");
		btnCancel.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				setVisible(false);
			}
		});
		actionPanel.add(btnCancel, "3, 1, 2, 1");

		JButton btnOk = new JButton("OK");
		btnOk.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				setVisible(false);
				new Thread() {
					@Override
					public void run() {
						saveConfiguration();
						client.startSynchronizing();
					}
				}.start();
			}
		});
		actionPanel.add(btnOk, "7, 1, 2, 1");

		loadConfiguration();
		enableSyncComponents(isSynchronizationEnabled());
		setFocus(btnCancel);
		setVisible(true);
	}

	private void loadConfiguration() {
		hostField.setText(InfologClientProperties.serverHostName);
		userField.setText(InfologClientProperties.username);
		passwordField.setText(JasyptCrypt.decryptPassword(InfologClientProperties.password));
		intervalSpinner.setValue(isSynchronizationEnabled() ? InfologClientProperties.pollInterval : DEFAULT_INTERVAL);
		portSpinner.setValue(InfologClientProperties.serverPort);
		httpsCBox.setSelected(InfologClientProperties.https);
		encryptionTypeBox.setSelectedItem(EncryptionType.getEncryptionType(InfologClientProperties.encryptionType));
		rdbtnServerWins.setSelected(InfologClientProperties.serverWins);
		rdbtnClientWins.setSelected(!InfologClientProperties.serverWins);
	}

	private boolean isSynchronizationEnabled() {
		return InfologClientProperties.pollInterval > 0;
	}

	private void syncEnbledChanged(JCheckBox box) {
		enableSyncComponents(box.isSelected());
	}

	private void enableSyncComponents(boolean enabled) {
		for (JComponent component : syncComponents) {
			component.setEnabled(enabled);
		}
	}

	private void setFocus(final JComponent component) {
		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				component.requestFocusInWindow();
			}
		});
	}

	private void tryLogin() {
		boolean success = client.tryRemoteLogin(hostField.getText(), (Integer) portSpinner.getValue(), userField
		   .getText(), new String(passwordField.getPassword()), httpsCBox.isSelected(),
		   (EncryptionType) encryptionTypeBox.getSelectedItem());
		if (success) {
			JOptionPane.showMessageDialog(this, LOGIN_SUCCESS, LOGIN_SUCCESS_TITLE, JOptionPane.INFORMATION_MESSAGE);
		} else {
			JOptionPane.showMessageDialog(this, LOGIN_FAILED, LOGIN_FAILED_TITLE, JOptionPane.ERROR_MESSAGE);
		}
	}

	private void saveConfiguration() {
		Properties properties = Properties.getInstance();
		properties.setProperty("serverhost", hostField.getText());
		InfologClientProperties.serverHostName = hostField.getText();
		properties.setProperty("serverport", portSpinner.getValue().toString());
		InfologClientProperties.serverPort = (Integer) portSpinner.getValue();
		properties.setProperty("username", userField.getText());
		InfologClientProperties.username = userField.getText();
		properties.setProperty("password", JasyptCrypt.encryptPassword(passwordField.getPassword()));
		InfologClientProperties.password = JasyptCrypt.encryptPassword(passwordField.getPassword());
		if (enableBox.isSelected()) {
			properties.setProperty("pollinterval", intervalSpinner.getValue().toString());
			InfologClientProperties.pollInterval = (Integer) intervalSpinner.getValue();
		} else {
			properties.setProperty("pollinterval", DISABLED_INTERVAL + "");
			InfologClientProperties.pollInterval = DISABLED_INTERVAL;
		}
		properties.setProperty("https", httpsCBox.isSelected() + "");
		InfologClientProperties.https = httpsCBox.isSelected();
		InfologClientProperties.encryptionType = encryptionTypeBox.getSelectedItem().toString();
		properties.setProperty("encryptionType", encryptionTypeBox.getSelectedItem().toString());
		InfologClientProperties.serverWins = rdbtnServerWins.isSelected();
		properties.setProperty("serverWins", rdbtnServerWins.isSelected() + "");
		properties.save();
	}

	public static void main(String[] args) throws ClassNotFoundException, InstantiationException,
	   IllegalAccessException, UnsupportedLookAndFeelException {
		UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		SyncConfigDialog dialog = new SyncConfigDialog(null, null);
		dialog.setVisible(true);
	}

}
