/*
 * Copyright 2013, RagingGoblin <http://raginggoblin.wordpress.com>
 * 
 * This file is part of Infolog.
 *
 *  Infolog is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Infolog is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Infolog.  If not, see <http://www.gnu.org/licenses/>.
 */

package raging.goblin.infolog.client.properties;

import org.apache.log4j.Logger;

public class PropertyLoader {

	private static Logger LOG = Logger.getLogger(PropertyLoader.class);

	public void loadProperties() {
		Properties properties = Properties.getInstance();

		InfologClientProperties.username = properties.getProperty("username", InfologClientProperties.username);
		InfologClientProperties.password = properties.getProperty("password", InfologClientProperties.password);
		InfologClientProperties.serverHostName = properties.getProperty("serverhost",
		   InfologClientProperties.serverHostName);
		InfologClientProperties.https = getBooleanProperty("https", InfologClientProperties.https);
		InfologClientProperties.keystore = properties.getProperty("keystore", InfologClientProperties.keystore);
		InfologClientProperties.passphrase = properties.getProperty("passphrase", InfologClientProperties.passphrase);
		InfologClientProperties.serverPort = getIntegerProperty("serverport", InfologClientProperties.serverPort);
		InfologClientProperties.pollInterval = getIntegerProperty("pollinterval", InfologClientProperties.pollInterval);
		InfologClientProperties.encryptionType = properties.getProperty("encryptionType",
		   InfologClientProperties.encryptionType);
		InfologClientProperties.serverWins = getBooleanProperty("serverWins", InfologClientProperties.serverWins);

		InfologClientProperties.trayIcon = properties.getProperty("systrayicon", InfologClientProperties.trayIcon);
		InfologClientProperties.doubleClickDelay = getIntegerProperty("doubleclickdelay",
		   InfologClientProperties.doubleClickDelay);
		InfologClientProperties.dragDelay = getIntegerProperty("dragdelay", InfologClientProperties.dragDelay);
		InfologClientProperties.spacing = getIntegerProperty("spacing", InfologClientProperties.spacing);
		InfologClientProperties.height = getIntegerProperty("height", InfologClientProperties.height);
		InfologClientProperties.width = getIntegerProperty("width", InfologClientProperties.width);
		InfologClientProperties.noteTopColor = getIntegerArrayProperty("notetopcolor",
		   InfologClientProperties.noteTopColor);
		InfologClientProperties.noteBottomColor = getIntegerArrayProperty("notebottomcolor",
		   InfologClientProperties.noteBottomColor);
		InfologClientProperties.taskTopColor = getIntegerArrayProperty("tasktopcolor",
		   InfologClientProperties.taskTopColor);
		InfologClientProperties.taskBottomColor = getIntegerArrayProperty("taskbottomcolor",
		   InfologClientProperties.taskBottomColor);
		InfologClientProperties.phoneTopColor = getIntegerArrayProperty("phonetopcolor",
		   InfologClientProperties.phoneTopColor);
		InfologClientProperties.phoneBottomColor = getIntegerArrayProperty("phonebottomcolor",
		   InfologClientProperties.phoneBottomColor);
		LOG.debug("Infolog client properties loaded");
	}

	private int getIntegerProperty(String key, int defaultValue) {
		Properties properties = Properties.getInstance();
		int result;
		try {
			result = Integer.parseInt(properties.getProperty(key, defaultValue + ""));
		} catch (Exception e) {
			LOG.warn("Unable to parse property: " + key + " with value: " + properties.getProperty(key));
			LOG.debug(e.getMessage(), e);
			result = defaultValue;
		}
		return result;
	}

	@SuppressWarnings("unused")
	private String[] getStringArrayProperty(String key, String[] defaultValue) {
		Properties properties = Properties.getInstance();
		StringBuilder stringDefault = new StringBuilder();
		for (String value : defaultValue) {
			stringDefault.append(value + ",");
		}
		String[] stringValues = properties.getProperty(key, stringDefault.substring(0, stringDefault.length() - 1))
		   .split(",");
		String[] result = new String[defaultValue.length];
		for (int i = 0; i < stringValues.length; i++) {
			result[i] = stringValues[i].trim();
		}
		return result;
	}

	private int[] getIntegerArrayProperty(String key, int[] defaultValue) {
		Properties properties = Properties.getInstance();
		StringBuilder stringDefault = new StringBuilder();
		for (int value : defaultValue) {
			stringDefault.append(value + ",");
		}
		String[] stringValues = properties.getProperty(key, stringDefault.substring(0, stringDefault.length() - 1))
		   .split(",");
		int[] result = new int[defaultValue.length];
		try {
			for (int i = 0; i < stringValues.length; i++) {
				result[i] = Integer.parseInt(stringValues[i].trim());
			}
		} catch (Exception e) {
			LOG.warn("Unable to parse property: " + key + " with value: " + properties.getProperty(key));
			LOG.debug(e.getMessage(), e);
			result = defaultValue;
		}
		return result;
	}

	private boolean getBooleanProperty(String key, boolean defaultValue) {
		Properties properties = Properties.getInstance();
		boolean result;
		try {
			result = Boolean.parseBoolean(properties.getProperty(key, defaultValue + ""));
		} catch (Exception e) {
			LOG.warn("Unable to parse property: " + key + " with value: " + properties.getProperty(key));
			LOG.debug(e.getMessage(), e);
			result = defaultValue;
		}
		return result;
	}
}
