/*
 * Copyright 2012, RagingGoblin <http://raginggoblin.wordpress.com>
 * 
 * This file is part of Infolog.
 *
 *  Infolog is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Infolog is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Infolog.  If not, see <http://www.gnu.org/licenses/>.
 */

package raging.goblin.infolog.server.data;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.OneToMany;
import javax.persistence.Table;

/**
 * Mapping class to Egroupware Accounts available in table egw_accounts.
 */
@Entity
@Table(name = "egw_accounts")
@NamedQueries({ @NamedQuery(name = "Account.findAll", query = "select a from Account a"),
    @NamedQuery(name = "Account.findByAccountId", query = "select a from Account a where a.accountId = :accountId"),
    @NamedQuery(name = "Account.findByAccountLid", query = "select a from Account a where a.accountLid = :accountLid") })
public class Account implements Serializable {

    private static final long serialVersionUID = 1L;
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Basic(optional = false)
    @Column(name = "account_id")
    private Integer accountId;
    @Column(name = "account_lid")
    private String accountLid;
    @Column(name = "account_pwd")
    private String accountPwd;
    @Column(name = "account_lastlogin")
    private Integer accountLastLogin;
    @Column(name = "account_lastloginfrom")
    private String accountLastLoginFrom;
    @Column(name = "account_lastpwd_change")
    private Integer accountLastPwChange;
    @Column(name = "account_status")
    private String accountStatus;
    @Column(name = "account_expires")
    private Integer accountExpires;
    @Column(name = "account_type")
    private String accountType;
    @Column(name = "account_primary_group")
    private Integer accountPrimaryGroup;
    @Column(name = "account_challenge")
    private String accountChallenge;
    @Column(name = "account_response")
    private String accountResponse;
    @OneToMany(mappedBy = "responsible", targetEntity = InfologDao.class, fetch = FetchType.EAGER)
    private Collection<InfologDao> responsibility;
    @OneToMany(mappedBy = "account", targetEntity = InfologDao.class, fetch = FetchType.EAGER)
    private Collection<InfologDao> infologs;

    public Account() {
    }

    public Account(Integer accountId) {
        this.accountId = accountId;
    }

    public int getAccountId() {
        return accountId;
    }

    public void setAccountId(Integer accountId) {
        this.accountId = accountId;
    }

    public String getAccountLid() {
        return accountLid;
    }

    public void setAccountLid(String accountLid) {
        this.accountLid = accountLid;
    }

    public String getAccountPwd() {
        return accountPwd;
    }

    public void setAccountPwd(String accountPwd) {
        this.accountPwd = accountPwd;
    }

    public int getAccountLastLogin() {
        return accountLastLogin;
    }

    public void setAccountLastLogin(Integer accountLastLogin) {
        this.accountLastLogin = accountLastLogin;
    }

    public String getAccountLastLoginFrom() {
        return accountLastLoginFrom;
    }

    public void setAccountLastLoginFrom(String accountLastLoginFrom) {
        this.accountLastLoginFrom = accountLastLoginFrom;
    }

    public int getAccountLastPwChange() {
        return accountLastPwChange;
    }

    public void setAccountLastPwChange(Integer accountLastPwChange) {
        this.accountLastPwChange = accountLastPwChange;
    }

    public String getAccountStatus() {
        return accountStatus;
    }

    public void setAccountStatus(String accountStatus) {
        this.accountStatus = accountStatus;
    }

    public int getAccountExpires() {
        return accountExpires;
    }

    public void setAccountExpires(Integer accountExpires) {
        this.accountExpires = accountExpires;
    }

    public String getAccountType() {
        return accountType;
    }

    public void setAccountType(String accountType) {
        this.accountType = accountType;
    }

    public int getAccountPrimaryGroup() {
        return accountPrimaryGroup;
    }

    public void setAccountPrimaryGroup(Integer accountPrimaryGroup) {
        this.accountPrimaryGroup = accountPrimaryGroup;
    }

    public String getAccountChallenge() {
        return accountChallenge;
    }

    public void setAccountChallenge(String accountChallenge) {
        this.accountChallenge = accountChallenge;
    }

    public String getAccountResponse() {
        return accountResponse;
    }

    public void setAccountResponse(String accountResponse) {
        this.accountResponse = accountResponse;
    }

    public Collection<InfologDao> getResponsibility() {
        return responsibility;
    }

    public void setResponsibility(Collection<InfologDao> responsibility) {
        this.responsibility = responsibility;
    }

    public Collection<InfologDao> getInfologs() {
        return infologs;
    }

    public void setInfologs(Collection<InfologDao> infologs) {
        this.infologs = infologs;
    }

    public List<InfologDao> getMyInfologs() {
        List<InfologDao> infologs = new ArrayList<InfologDao>();

        // I am the owner and no one is responsible
        for (InfologDao infolog : getInfologs()) {
            if (infolog.getInfoResponsible() == null) {
                infolog.setAccount(null);
                infolog.setInfoResponsible(null);
                infologs.add(infolog);
            }
        }

        // I am responsible
        for (InfologDao responsible : getResponsibility()) {
            responsible.setAccount(null);
            responsible.setInfoResponsible(null);
            infologs.add(responsible);
        }
        return infologs;
    }
}
