/*
 * Copyright 2012, RagingGoblin <http://raginggoblin.wordpress.com>
 * 
 * This file is part of Infolog.
 *
 *  Infolog is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Infolog is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Infolog.  If not, see <http://www.gnu.org/licenses/>.
 */

package raging.goblin.infolog.server.data;

import java.io.Serializable;
import java.math.BigInteger;

import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Table;

import org.hibernate.annotations.NotFound;
import org.hibernate.annotations.NotFoundAction;

/**
 * Mapping class to the Egroupware Infolog available in table egw_infolog.
 */
@Entity
@Table(name = "egw_infolog")
@NamedQueries({
    @NamedQuery(name = "InfologDao.findAll", query = "select e from InfologDao e"),
    @NamedQuery(name = "InfologDao.findByInfoId", query = "select e from InfologDao e where e.infoId = :id"),
    @NamedQuery(name = "InfologDao.findByInfoType", query = "select e from InfologDao e where e.infoType = :type"),
    @NamedQuery(name = "InfologDao.findByInfoFrom", query = "select e from InfologDao e where e.infoFrom = :infoFrom"),
    @NamedQuery(name = "InfologDao.findByInfoAddr", query = "select e from InfologDao e where e.infoAddr = :infoAddr"),
    @NamedQuery(
        name = "InfologDao.findByInfoSubject",
        query = "select e from InfologDao e where e.infoSubject = :subject"),
    @NamedQuery(name = "InfologDao.findByInfoAccess", query = "select e from InfologDao e where e.infoAccess = :infoAccess"),
    @NamedQuery(name = "InfologDao.findByInfoCat", query = "select e from InfologDao e where e.infoCat = :infoCat"),
    @NamedQuery(
        name = "InfologDao.findByInfoDatemodified",
        query = "select e from InfologDao e where e.infoDatemodified = :datemodified"),
    @NamedQuery(
        name = "InfologDao.findByInfoStartdate",
        query = "select e from InfologDao e where e.infoStartdate = :infoStartdate"),
    @NamedQuery(
        name = "InfologDao.findByInfoEnddate",
        query = "select e from InfologDao e where e.infoEnddate = :infoEnddate"),
    @NamedQuery(
        name = "InfologDao.findByInfoIdParent",
        query = "select e from InfologDao e where e.infoIdParent = :infoIdParent"),
    @NamedQuery(
        name = "InfologDao.findByInfoPlannedTime",
        query = "select e from InfologDao e where e.infoPlannedTime = :infoPlannedTime"),
    @NamedQuery(
        name = "InfologDao.findByInfoReplannedTime",
        query = "select e from InfologDao e where e.infoReplannedTime = :infoReplannedTime"),
    @NamedQuery(
        name = "InfologDao.findByInfoUsedTime",
        query = "select e from InfologDao e where e.infoUsedTime = :infoUsedTime"),
    @NamedQuery(name = "InfologDao.findByInfoStatus", query = "select e from InfologDao e where e.infoStatus = :infoStatus"),
    @NamedQuery(
        name = "InfologDao.findByInfoConfirm",
        query = "select e from InfologDao e where e.infoConfirm = :infoConfirm"),
    @NamedQuery(
        name = "InfologDao.findByInfoModifier",
        query = "select e from InfologDao e where e.infoModifier = :infoModifier"),
    @NamedQuery(name = "InfologDao.findByInfoLinkId", query = "select e from InfologDao e where e.infoLinkId = :infoLinkId"),
    @NamedQuery(
        name = "InfologDao.findByInfoPriority",
        query = "select e from InfologDao e where e.infoPriority = :infoPriority"),
    @NamedQuery(name = "InfologDao.findByPlId", query = "select e from InfologDao e where e.plId = :plId"),
    @NamedQuery(name = "InfologDao.findByInfoPrice", query = "select e from InfologDao e where e.infoPrice = :infoPrice"),
    @NamedQuery(
        name = "InfologDao.findByInfoPercent",
        query = "select e from InfologDao e where e.infoPercent = :infoPercent"),
    @NamedQuery(
        name = "InfologDao.findByInfoDatecompleted",
        query = "select e from InfologDao e where e.infoDatecompleted = :infoDatecompleted"),
    @NamedQuery(
        name = "InfologDao.findByInfoLocation",
        query = "select e from InfologDao e where e.infoLocation = :infoLocation"),
    @NamedQuery(
        name = "InfologDao.findByInfoCustomFrom",
        query = "select e from InfologDao e where e.infoCustomFrom = :infoCustomFrom"),
    @NamedQuery(name = "InfologDao.findByInfoUid", query = "select e from InfologDao e where e.infoUid = :infoUid") })
public class InfologDao implements Serializable {

    private static final long serialVersionUID = 1L;
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    @Basic(optional = false)
    @Column(name = "info_id")
    private Integer infoId;
    @Basic(optional = false)
    @Column(name = "info_type")
    private String infoType;
    @Column(name = "info_from")
    private String infoFrom;
    @Column(name = "info_addr")
    private String infoAddr;
    @Basic(optional = false)
    @Column(name = "info_subject")
    private String infoSubject;
    @Lob
    @Column(name = "info_des")
    private String infoDes;
    @Column(name = "info_access")
    private String infoAccess;
    @Basic(optional = false)
    @Column(name = "info_cat")
    private Integer infoCat;
    @Basic(optional = false)
    @Column(name = "info_datemodified")
    private Long infoDatemodified;
    @Basic(optional = false)
    @Column(name = "info_startdate")
    private Long infoStartdate;
    @Basic(optional = false)
    @Column(name = "info_enddate")
    private Long infoEnddate;
    @Basic(optional = false)
    @Column(name = "info_id_parent")
    private Integer infoIdParent;
    @Basic(optional = false)
    @Column(name = "info_planned_time")
    private Integer infoPlannedTime;
    @Basic(optional = false)
    @Column(name = "info_replanned_time")
    private Integer infoReplannedTime;
    @Basic(optional = false)
    @Column(name = "info_used_time")
    private Integer infoUsedTime;
    @Column(name = "info_status")
    private String infoStatus;
    @Column(name = "info_confirm")
    private String infoConfirm;
    @Basic(optional = false)
    @Column(name = "info_modifier")
    private Integer infoModifier;
    @Basic(optional = false)
    @Column(name = "info_link_id")
    private Integer infoLinkId;
    @Column(name = "info_priority")
    private Short infoPriority;
    @Column(name = "pl_id")
    private Integer plId;
    @Column(name = "info_price")
    private Double infoPrice;
    @Column(name = "info_percent")
    private Short infoPercent;
    @Column(name = "info_datecompleted")
    private Long infoDatecompleted;
    @Column(name = "info_location")
    private String infoLocation;
    @Column(name = "info_custom_from")
    private Short infoCustomFrom;
    @Column(name = "info_uid")
    private String infoUid;
    @Column(name = "info_cc")
    private String infoCc;
    @Column(name = "caldav_name")
    private String caldavName;
    @Column(name = "info_etag")
    private Integer infoEtag;
    @Column(name = "info_created")
    private BigInteger infoCreated;
    @Column(name = "info_creator")
    private Integer infoCreator;
    @ManyToOne(optional = true)
    @NotFound(action = NotFoundAction.IGNORE)
    @JoinColumn(name = "info_responsible")
    private Account responsible;
    @ManyToOne(optional = false)
    @JoinColumn(name = "info_owner")
    private Account account;

    public InfologDao() {
    }

    public InfologDao(Integer infoId) {
        this.infoId = infoId;
    }

    public InfologDao(String infoType, int infoCat, long infoDatemodified, long infoStartdate, long infoEnddate,
        int infoIdParent, int infoPlannedTime, int infoReplannedTime, int infoUsedTime, int infoModifier, int infoLinkId) {
        this.infoType = infoType;
        this.infoCat = infoCat;
        this.infoDatemodified = infoDatemodified;
        this.infoStartdate = infoStartdate;
        this.infoEnddate = infoEnddate;
        this.infoIdParent = infoIdParent;
        this.infoPlannedTime = infoPlannedTime;
        this.infoReplannedTime = infoReplannedTime;
        this.infoUsedTime = infoUsedTime;
        this.infoModifier = infoModifier;
        this.infoLinkId = infoLinkId;
    }

    public InfologDao(Integer infoId, String infoType, int infoCat, long infoDatemodified, long infoStartdate,
        long infoEnddate, int infoIdParent, int infoPlannedTime, int infoReplannedTime, int infoUsedTime,
        int infoModifier, int infoLinkId) {
        this.infoId = infoId;
        this.infoType = infoType;
        this.infoCat = infoCat;
        this.infoDatemodified = infoDatemodified;
        this.infoStartdate = infoStartdate;
        this.infoEnddate = infoEnddate;
        this.infoIdParent = infoIdParent;
        this.infoPlannedTime = infoPlannedTime;
        this.infoReplannedTime = infoReplannedTime;
        this.infoUsedTime = infoUsedTime;
        this.infoModifier = infoModifier;
        this.infoLinkId = infoLinkId;
    }

    public Integer getInfoId() {
        return infoId;
    }

    public void setInfoId(Integer infoId) {
        this.infoId = infoId;
    }

    public String getInfoType() {
        return infoType;
    }

    public void setInfoType(String infoType) {
        this.infoType = infoType;
    }

    public String getInfoFrom() {
        return infoFrom;
    }

    public void setInfoFrom(String infoFrom) {
        this.infoFrom = infoFrom;
    }

    public String getInfoAddr() {
        return infoAddr;
    }

    public void setInfoAddr(String infoAddr) {
        this.infoAddr = infoAddr;
    }

    public String getInfoSubject() {
        return infoSubject;
    }

    public void setInfoSubject(String infoSubject) {
        this.infoSubject = infoSubject;
    }

    public String getInfoDes() {
        return infoDes;
    }

    public void setInfoDes(String infoDes) {
        this.infoDes = infoDes;
    }

    public Account getInfoResponsible() {
        return responsible;
    }

    public void setInfoResponsible(Account infoResponsible) {
        this.responsible = infoResponsible;
    }

    public String getInfoAccess() {
        return infoAccess;
    }

    public void setInfoAccess(String infoAccess) {
        this.infoAccess = infoAccess;
    }

    public Integer getInfoCat() {
        return infoCat;
    }

    public void setInfoCat(Integer infoCat) {
        this.infoCat = infoCat;
    }

    public Long getInfoDatemodified() {
        return infoDatemodified;
    }

    public void setInfoDatemodified(Long infoDatemodified) {
        this.infoDatemodified = infoDatemodified;
    }

    public Long getInfoStartdate() {
        return infoStartdate;
    }

    public void setInfoStartdate(Long infoStartdate) {
        this.infoStartdate = infoStartdate;
    }

    public Long getInfoEnddate() {
        return infoEnddate;
    }

    public void setInfoEnddate(Long infoEnddate) {
        this.infoEnddate = infoEnddate;
    }

    public Integer getInfoIdParent() {
        return infoIdParent;
    }

    public void setInfoIdParent(Integer infoIdParent) {
        this.infoIdParent = infoIdParent;
    }

    public Integer getInfoPlannedTime() {
        return infoPlannedTime;
    }

    public void setInfoPlannedTime(Integer infoPlannedTime) {
        this.infoPlannedTime = infoPlannedTime;
    }

    public Integer getInfoReplannedTime() {
        return infoReplannedTime;
    }

    public void setInfoReplannedTime(Integer infoReplannedTime) {
        this.infoReplannedTime = infoReplannedTime;
    }

    public Integer getInfoUsedTime() {
        return infoUsedTime;
    }

    public void setInfoUsedTime(Integer infoUsedTime) {
        this.infoUsedTime = infoUsedTime;
    }

    public String getInfoStatus() {
        return infoStatus;
    }

    public void setInfoStatus(String infoStatus) {
        this.infoStatus = infoStatus;
    }

    public String getInfoConfirm() {
        return infoConfirm;
    }

    public void setInfoConfirm(String infoConfirm) {
        this.infoConfirm = infoConfirm;
    }

    public Integer getInfoModifier() {
        return infoModifier;
    }

    public void setInfoModifier(Integer infoModifier) {
        this.infoModifier = infoModifier;
    }

    public Integer getInfoLinkId() {
        return infoLinkId;
    }

    public void setInfoLinkId(Integer infoLinkId) {
        this.infoLinkId = infoLinkId;
    }

    public Short getInfoPriority() {
        return infoPriority;
    }

    public void setInfoPriority(Short infoPriority) {
        this.infoPriority = infoPriority;
    }

    public Integer getPlId() {
        return plId;
    }

    public void setPlId(Integer plId) {
        this.plId = plId;
    }

    public Double getInfoPrice() {
        return infoPrice;
    }

    public void setInfoPrice(Double infoPrice) {
        this.infoPrice = infoPrice;
    }

    public Short getInfoPercent() {
        return infoPercent;
    }

    public void setInfoPercent(Short infoPercent) {
        this.infoPercent = infoPercent;
    }

    public Long getInfoDatecompleted() {
        return infoDatecompleted;
    }

    public void setInfoDatecompleted(Long infoDatecompleted) {
        this.infoDatecompleted = infoDatecompleted;
    }

    public String getInfoLocation() {
        return infoLocation;
    }

    public void setInfoLocation(String infoLocation) {
        this.infoLocation = infoLocation;
    }

    public Short getInfoCustomFrom() {
        return infoCustomFrom;
    }

    public void setInfoCustomFrom(Short infoCustomFrom) {
        this.infoCustomFrom = infoCustomFrom;
    }

    public String getInfoUid() {
        return infoUid;
    }

    public void setInfoUid(String infoUid) {
        this.infoUid = infoUid;
    }

    public String getInfoCc() {
        return infoCc;
    }

    public void setInfoCc(String infoCc) {
        this.infoCc = infoCc;
    }

    public String getCaldavName() {
        return caldavName;
    }

    public void setCaldavName(String caldavName) {
        this.caldavName = caldavName;
    }

    public Integer getInfoEtag() {
        return infoEtag;
    }

    public void setInfoEtag(Integer infoEtag) {
        this.infoEtag = infoEtag;
    }

    public BigInteger getInfoCreated() {
        return infoCreated;
    }

    public void setInfoCreated(BigInteger infoCreated) {
        this.infoCreated = infoCreated;
    }

    public Integer getInfoCreator() {
        return infoCreator;
    }

    public void setInfoCreator(Integer infoCreator) {
        this.infoCreator = infoCreator;
    }

    public Account getAccount() {
        return account;
    }

    public void setAccount(Account account) {
        this.account = account;
    }

    public void update(InfologDao infolog) {
        infoId = infolog.getInfoId();
        infoType = infolog.getInfoType();
        infoFrom = infolog.getInfoFrom();
        infoAddr = infolog.getInfoAddr();
        infoSubject = infolog.getInfoSubject();
        infoDes = infolog.getInfoDes();
        infoAccess = infolog.getInfoAccess();
        infoCat = infolog.getInfoCat();
        infoDatemodified = infolog.getInfoDatemodified();
        infoStartdate = infolog.getInfoStartdate();
        infoEnddate = infolog.getInfoEnddate();
        infoIdParent = infolog.getInfoIdParent();
        infoPlannedTime = infolog.getInfoPlannedTime();
        infoReplannedTime = infolog.getInfoReplannedTime();
        infoUsedTime = infolog.getInfoUsedTime();
        infoStatus = infolog.getInfoStatus();
        infoConfirm = infolog.getInfoConfirm();
        infoModifier = infolog.getInfoModifier();
        infoLinkId = infolog.getInfoLinkId();
        infoPriority = infolog.getInfoPriority();
        plId = infolog.getPlId();
        infoPrice = infolog.getInfoPrice();
        infoPercent = infolog.getInfoPercent();
        infoDatecompleted = infolog.getInfoDatecompleted();
        infoLocation = infolog.getInfoLocation();
        infoCustomFrom = infolog.getInfoCustomFrom();
        infoUid = infolog.getInfoUid();
        responsible = infolog.getInfoResponsible();
        account = infolog.getAccount();
    }

    @Override
    public int hashCode() {
        return infoId.hashCode();
    }

    @Override
    public boolean equals(Object object) {
        if (object instanceof InfologDao) {
            return infoId.equals(((InfologDao) object).infoId);
        }
        return false;
    }

    @Override
    public String toString() {
        return "infolog.EgwInfolog[id=" + infoId + "]";
    }
}
