/*
 * Copyright 2012, RagingGoblin <http://raginggoblin.wordpress.com>
 * 
 * This file is part of Infolog.
 *
 *  Infolog is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Infolog is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Infolog.  If not, see <http://www.gnu.org/licenses/>.
 */

package raging.goblin.infolog.server.gui;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.SpinnerNumberModel;
import javax.swing.UIManager;
import javax.swing.UIManager.LookAndFeelInfo;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.text.BadLocationException;
import javax.swing.text.StyledDocument;

import org.apache.log4j.Logger;

import raging.goblin.infolog.server.ServerApplication;

/**
 * Provides the raging.goblin.infolog.server.gui for the Infolog Server.
 */
public class ServerGui extends JFrame {

    private static Logger LOG = Logger.getLogger(ServerGui.class);

    private static final String SCREENTITLE = "Infolog server";
    private static final String PORT_LABEL = "Port: ";
    private static final String ENDPOINT_LABEL = "Endpoint IP: ";

    private ServerApplication application;
    private JButton startButton;
    private JButton stopButton;
    private StyledDocument messages;
    private SpinnerNumberModel portSpinnerModel;
    private JTextField ipField;

    public ServerGui(ServerApplication application) {
        super(ServerGui.SCREENTITLE);
        this.application = application;

        loadLaf();
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setSize(400, 200);
        initGui();
    }

    private void initGui() {
        JPanel buttonPanel = new JPanel(new GridLayout(3, 0, 10, 10));

        // Start button
        startButton = new JButton("Start");
        startButton.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent event) {
                String message;
                ServerApplication.ipAddress = ipField.getText();
                ServerApplication.port = portSpinnerModel.getNumber().intValue();
                if (application.startServer()) {
                    message = "Server started by raging.goblin.infolog.server.gui";
                    LOG.info(message);
                    stopButton.setEnabled(true);
                    startButton.setEnabled(false);
                } else {
                    message = "Server could not be started, see log for details";
                }
                setMessage(message);
            }
        });
        buttonPanel.add(startButton);

        // Stop button
        stopButton = new JButton("Stop");
        stopButton.setEnabled(false);
        stopButton.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent event) {
                application.stopServer();
                String message = "Server stopped by raging.goblin.infolog.server.gui";
                LOG.info(message);
                stopButton.setEnabled(false);
                startButton.setEnabled(true);
                setMessage(message);
            }
        });
        buttonPanel.add(stopButton);

        // IP textfield and label
        buttonPanel.add(new JLabel(ENDPOINT_LABEL));
        ipField = new JTextField(ServerApplication.ipAddress);
        buttonPanel.add(ipField);

        // Port spinner and label
        buttonPanel.add(new JLabel(PORT_LABEL));
        portSpinnerModel = new SpinnerNumberModel(ServerApplication.port, 1, 65535, 1);
        JSpinner portSpinner = new JSpinner(portSpinnerModel);
        buttonPanel.add(portSpinner);

        // Message panel
        JTextPane messagePane = new JTextPane();
        messages = messagePane.getStyledDocument();

        getContentPane().add(buttonPanel, BorderLayout.NORTH);
        getContentPane().add(messagePane, BorderLayout.CENTER);
    }

    private void setMessage(String message) {
        try {
            messages.insertString(messages.getLength(), message + "\n", null);
        } catch (BadLocationException e) {
            e.printStackTrace();
        }
    }

    private void loadLaf() {
        try {
            if (UIManager.getSystemLookAndFeelClassName().contains("Metal")) {
                setLinuxLaf();
            } else {
                UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
            }
        } catch (Exception e) {
            LOG.error("Could not set system look and feel");
            LOG.debug(e.getMessage(), e);
        }
    }

    private void setLinuxLaf() throws ClassNotFoundException, InstantiationException, IllegalAccessException,
        UnsupportedLookAndFeelException {
        try {
            UIManager.setLookAndFeel("com.sun.java.swing.plaf.gtk.GTKLookAndFeel");
        } catch (Exception e) {
            setNimbusLaf();
        }
    }

    private void setNimbusLaf() throws ClassNotFoundException, InstantiationException, IllegalAccessException,
        UnsupportedLookAndFeelException {
        for (LookAndFeelInfo info : UIManager.getInstalledLookAndFeels()) {
            if ("Nimbus".equals(info.getName())) {
                UIManager.setLookAndFeel(info.getClassName());
                break;
            }
        }
    }
}
