/*
 * Copyright 2013, RagingGoblin <http://raginggoblin.wordpress.com>
 * 
 * This file is part of Infolog.
 *
 *  Infolog is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Infolog is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Infolog.  If not, see <http://www.gnu.org/licenses/>.
 */

package raging.goblin.infolog.server.webservice;

import java.net.URI;
import java.util.List;

import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.UriBuilder;

import org.apache.log4j.Logger;
import org.glassfish.grizzly.http.server.HttpServer;
import org.glassfish.jersey.grizzly2.httpserver.GrizzlyHttpServerFactory;
import org.glassfish.jersey.jackson.JacksonFeature;
import org.glassfish.jersey.server.ResourceConfig;

import raging.goblin.infolog.server.ServerApplication;
import raging.goblin.infolog.server.data.InfologStorage;
import raging.goblin.infolog.sync.data.Infolog;
import raging.goblin.infolog.sync.webserviceclient.InfologWebRequest;

@Path("/infolog")
public class InfologWebservice {

    private static final Logger LOG = Logger.getLogger(InfologWebservice.class);

    private HttpServer httpServer;

    public InfologWebservice() {
        // Grizzly :)
    }

    public boolean start() {
        URI uri = UriBuilder.fromUri("http://" + ServerApplication.ipAddress).port(ServerApplication.port).build();
        try {
            httpServer = GrizzlyHttpServerFactory.createHttpServer(uri, new InfologResourceConfig());
            LOG.info("Server started at http://" + ServerApplication.ipAddress + ":" + ServerApplication.port
                + "/infolog/");
            return true;
        } catch (Exception e) {
            LOG.error("Error starting server", e);
        }
        return false;
    }

    public void stop() {
        httpServer.stop();
        LOG.info("Server stopped");
    }

    @GET
    @Path("/authenticate")
    @Produces(MediaType.TEXT_PLAIN)
    public String authenticate(@QueryParam("username") String username, @QueryParam("password") String password,
        @QueryParam("encryptionType") String encryptionType) {
        return "" + InfologStorage.getInstance().authenticate(username, password, encryptionType);
    }

    @POST
    @Path("/updateInfologs")
    @Consumes(MediaType.APPLICATION_JSON)
    public void updateInfologs(InfologWebRequest request) {
        InfologStorage.getInstance().updateInfologs(request.getInfologs(), request.getUsername(),
            request.getPassword(), request.getEncryptionType());
    }

    @POST
    @Path("/addInfologs")
    @Consumes(MediaType.APPLICATION_JSON)
    @Produces(MediaType.APPLICATION_JSON)
    public List<Infolog> addInfologs(InfologWebRequest request) {
        return InfologStorage.getInstance().addInfologs(request.getInfologs(), request.getUsername(),
            request.getPassword(), request.getEncryptionType());
    }

    @POST
    @Path("/removeInfologs")
    @Consumes(MediaType.APPLICATION_JSON)
    public void removeInfologs(InfologWebRequest request) {
        InfologStorage.getInstance().removeInfologs(request.getInfologs(), request.getUsername(),
            request.getPassword(), request.getEncryptionType());
    }

    @GET
    @Path("/getInfologs")
    @Produces(MediaType.APPLICATION_JSON)
    public List<Infolog> getInfologs(@QueryParam("username") String username, @QueryParam("password") String password,
        @QueryParam("encryptionType") String encryptionType) {
        return InfologStorage.getInstance().getInfologs(username, password, encryptionType);
    }

    private class InfologResourceConfig extends ResourceConfig {
        public InfologResourceConfig() {
            super(InfologWebservice.class, JacksonFeature.class);
        }
    }
}
