/*
 * Copyright 2013, RagingGoblin <http://raginggoblin.wordpress.com>
 * 
 * This file is part of Infolog.
 *
 *  Infolog is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Infolog is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Infolog.  If not, see <http://www.gnu.org/licenses/>.
 */

package raging.goblin.infolog.sync;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URISyntaxException;
import java.util.List;

import org.apache.http.client.ClientProtocolException;
import org.codehaus.jackson.JsonGenerationException;
import org.codehaus.jackson.JsonParseException;
import org.codehaus.jackson.map.JsonMappingException;

import raging.goblin.infolog.sync.data.Infolog;
import raging.goblin.infolog.sync.webserviceclient.HttpErrorCodeException;

/**
 * Infolog storage for {@link Infolog}s this can either be a remote storage through a webservice or a local storage.
 */
public interface InfologStorage {

    public boolean authenticate(String username, String password, String encryptionType)
        throws ClientProtocolException, IOException, HttpErrorCodeException, URISyntaxException;

    public void updateInfologs(List<Infolog> infologs, String username, String password, String encryptionType)
        throws JsonGenerationException, JsonMappingException, UnsupportedEncodingException, URISyntaxException,
        IOException, HttpErrorCodeException;

    public List<Infolog> addInfologs(List<Infolog> infologs, String username, String password, String encryptionType)
        throws JsonParseException, JsonMappingException, IOException, URISyntaxException, HttpErrorCodeException;

    public void removeInfologs(List<Infolog> infologs, String username, String password, String encryptionType)
        throws JsonGenerationException, JsonMappingException, UnsupportedEncodingException, URISyntaxException,
        IOException, HttpErrorCodeException;

    public List<Infolog> getInfologs(String username, String password, String encryptionType)
        throws JsonParseException, JsonMappingException, IOException, HttpErrorCodeException, URISyntaxException;

}
