package raging.goblin.infolog.sync.webserviceclient;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.URISyntaxException;
import java.util.List;

import org.apache.http.HttpResponse;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.codehaus.jackson.JsonGenerationException;
import org.codehaus.jackson.JsonParseException;
import org.codehaus.jackson.map.JsonMappingException;
import org.codehaus.jackson.map.ObjectMapper;
import org.codehaus.jackson.type.TypeReference;

import raging.goblin.infolog.sync.InfologStorage;
import raging.goblin.infolog.sync.data.Infolog;

public class WebserviceClient implements InfologStorage {

    private String ipAddress;
    private int port;
    private boolean useHttps;
    private DefaultHttpClient httpClient;
    private ObjectMapper mapper;

    public WebserviceClient(String ipAddress, int port, boolean useHttps) {
        this.ipAddress = ipAddress;
        this.port = port;
        this.useHttps = useHttps;
        mapper = new ObjectMapper();
    }

    public void start(DefaultHttpClient httpClient) {
        this.httpClient = httpClient;
    }

    public void stop() {
        if (httpClient != null) {
            httpClient.getConnectionManager().shutdown();
        }
    }

    @Override
    public boolean authenticate(String username, String password, String encryptionType)
        throws ClientProtocolException, IOException, HttpErrorCodeException, URISyntaxException {
        HttpResponse response = getRequest("authenticate", "username=" + username, "password=" + password,
            "encryptionType=" + encryptionType);
        String value = getResponseContent(response);
        return Boolean.valueOf(value);
    }

    @Override
    public void updateInfologs(List<Infolog> infologs, String username, String password, String encryptionType)
        throws JsonGenerationException, JsonMappingException, UnsupportedEncodingException, URISyntaxException,
        IOException, HttpErrorCodeException {
        postRequest("updateInfologs", new InfologWebRequest(username, password, encryptionType, infologs));
    }

    @Override
    public List<Infolog> addInfologs(List<Infolog> infologs, String username, String password, String encryptionType)
        throws JsonParseException, JsonMappingException, IOException, URISyntaxException, HttpErrorCodeException {
        HttpResponse response = postRequest("addInfologs", new InfologWebRequest(username, password, encryptionType,
            infologs));
        String value = getResponseContent(response);
        return mapper.readValue(value, new TypeReference<List<Infolog>>() {
        });
    }

    @Override
    public void removeInfologs(List<Infolog> infologs, String username, String password, String encryptionType)
        throws JsonGenerationException, JsonMappingException, UnsupportedEncodingException, URISyntaxException,
        IOException, HttpErrorCodeException {
        postRequest("removeInfologs", new InfologWebRequest(username, password, encryptionType, infologs));
    }

    @Override
    public List<Infolog> getInfologs(String username, String password, String encryptionType)
        throws JsonParseException, JsonMappingException, IOException, HttpErrorCodeException, URISyntaxException {
        HttpResponse response = getRequest("getInfologs", "username=" + username, "password=" + password,
            "encryptionType=" + encryptionType);
        String value = getResponseContent(response);
        return mapper.readValue(value, new TypeReference<List<Infolog>>() {
        });
    }

    private String getResponseContent(HttpResponse response) throws IllegalStateException, IOException {
        BufferedReader in = new BufferedReader(new InputStreamReader((response.getEntity().getContent())));
        String line;
        StringBuilder content = new StringBuilder();
        while ((line = in.readLine()) != null) {
            content.append(line);
        }
        return content.toString();
    }

    private HttpResponse postRequest(String methodName, InfologWebRequest infologWebRequest) throws URISyntaxException,
        JsonGenerationException, JsonMappingException, UnsupportedEncodingException, IOException,
        HttpErrorCodeException {
        HttpPost postRequest = new HttpPost(getUri(methodName));
        StringEntity input = new StringEntity(mapper.writeValueAsString(infologWebRequest));
        input.setContentType("application/json");
        postRequest.setEntity(input);
        HttpResponse response = httpClient.execute(postRequest);

        if (response.getStatusLine().getStatusCode() != 200 && response.getStatusLine().getStatusCode() != 204) {
            throw new HttpErrorCodeException("Failed to retrieve valid content: HTTP error code: "
                + response.getStatusLine().getStatusCode());
        }

        return response;
    }

    private HttpResponse getRequest(String methodName, String... parameters) throws ClientProtocolException,
        IOException, HttpErrorCodeException, URISyntaxException {
        HttpGet request = new HttpGet(getUri(methodName, parameters));
        HttpResponse response = httpClient.execute(request);

        if (response.getStatusLine().getStatusCode() != 200) {
            throw new HttpErrorCodeException("Failed to retrieve valid content: HTTP error code: "
                + response.getStatusLine().getStatusCode());
        }
        return response;
    }

    private String getUri(String methodName, String... parameters) throws URISyntaxException {
        String https = useHttps ? "s" : "";
        StringBuilder uri = new StringBuilder("http" + https + "://" + ipAddress + ":" + port + "/infolog/"
            + methodName);
        if (parameters.length != 0) {
            uri.append("?");
            for (int i = 0; i < parameters.length - 1; i++) {
                uri.append(parameters[i] + "&");
            }
            uri.append(parameters[parameters.length - 1]);
        }
        return uri.toString();
    }
}
